<?php
/**
 * @file Admin stuff for mailmain_manager module
 */

/**
 * Admin display of mailman lists.
 */
function _mailman_manager_admin_display() {
  if (!user_access('administer mailman_manager')) {
    return '';
  }
  $output = '';

  $headers = array(
    array('data' => t('Name'), 'field' => 'name'),
    array('data' => t('Request Address'), 'field' => 'command'),
    t('Roles'),
    array('data' => t('Actions')),
  );

  $num_per_page = 15;
  $result = pager_query('SELECT * FROM {mailman_lists} '. tablesort_sql($headers), $num_per_page);
  $lists = array();

  //get the roles from the database
  $all_roles = _mailman_manager_get_roles();
  $rows = array();
  while ($list = db_fetch_object($result)) {
    //the links/actions for this row
    $actions = l(t('Edit'), 'admin/settings/mailman_manager/edit/'. $list->lid) .'<br />'. l(t('Delete'), 'admin/settings/mailman_manager/delete/'. $list->lid);
    $roles = unserialize($list->roles);
    $list->role_names = array();
    foreach ($roles as $rid => $value) {

      if ($value) {
        $list->role_names[] = $all_roles[$rid];
      }
    }

    $lists[] = array(
      $list->name,
      $list->command,
      implode($list->role_names, ', '),
      $actions
    );

  }

  if (count($lists) == 0) {
    $output .= '<p>'. t('No lists.') .'</p>';
  }
  $output .= l(t('Add new mailing list'), 'admin/settings/mailman_manager/add');
  if (count($lists)) {
    $output .= theme('table', $headers, $lists);
    $output .= theme('pager');
  }


  return $output;
}


function mailman_manager_admin_form(&$form_state, $lid = NULL) {
  $form = array();
  if ($lid) {
    $list = mailman_manager_get_list($lid);
    $form['lid'] = array(
      '#type' => 'hidden',
      '#value' => $lid,
    );
  }

  $form['name'] = array(
    '#title' => 'Mailing List name',
    '#default_value' => !empty($list->name) ? $list->name : '',
    '#type' => 'textfield',
    '#required' => TRUE,
  );
  $form['reqaddress'] = array(
    '#title' => t("Mailing List 'request' address"),
    '#default_value' => !empty($list->command) ? $list->command : '',
    '#description' => t('Commands are sent here'),
    '#type' => 'textfield',
    '#required' => TRUE,
  );
  $form['adminaddress'] = array(
    '#title' => t("Mailing List 'admin' address"),
    '#default_value' => !empty($list->admin) ? $list->admin : '',
    '#description' => t('(Optional. admin is used for BCC)'),
    '#type' => 'textfield',
  );
  $form['webaddress'] = array(
    '#title' => t('Mailing list web address for users'),
    '#default_value' => !empty($list->web) ? $list->web : '',
    '#description' => t('http://...' ) . t('Leave empty if hidden.'),
    '#type' => 'textfield',
  );
  $form['webarchive'] = array(
    '#title' => t('Mailing list web archive address for users'),
    '#default_value' => !empty($list->webarch) ? $list->webarch : '',
    '#description' => t('http://.... ') . t('Leave empty if hidden.'),
    '#type' => 'textfield',
  );


  $form['roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Roles'),
    '#description' => t('Only offer to users in these roles.'),
    '#options' => _mailman_manager_get_roles(),
  );
  if (isset($list->roles)) {
     $form['roles']['#default_value'] = unserialize($list->roles);
  }

  $form['Submit'] = array(
    '#type' => 'submit',
    '#value' => ($lid ? t('Save') : t('Add')),
  );

  return $form;
}


function mailman_manager_admin_form_validate($form, &$form_state) {
  // Check name is unique
  $lid = !empty($form_state['values']['lid']) ? $form_state['values']['lid'] : 0;
  $result = db_query("SELECT COUNT(*) from {mailman_lists} WHERE name = '%s'". ($lid ? ' AND lid <> %d' : ''), $form_state['values']['name'], $lid);
  $num_rows = db_result($result);
  if ($num_rows != '0') {
    form_set_error('name', 'Name already in use');
  }
  if ($form_state['values']['reqaddress'] && !valid_email_address($form_state['values']['reqaddress'])) {
    form_set_error('reqaddress', t('"%reqaddress" is not a valid email address', array('%reqaddress' => $form_state['values']['reqaddress'])));
  }
  if ($form_state['values']['adminaddress'] && !valid_email_address($form_state['values']['adminaddress'])) {
    form_set_error('adminaddress', t('"%adminaddress" is not a valid email address', array('%adminaddress' => $form_state['values']['adminaddress'])));
  }
  if ($form_state['values']['webaddress'] && !valid_url($form_state['values']['webaddress'])) {
    form_set_error('webaddress', t('"%webaddress" is not a valid url', array('%webaddress' => $form_state['values']['webaddress'])));
  }
  if ($form_state['values']['webarchive'] && !valid_url($form_state['values']['webarchive'])) {
    form_set_error('webarchive', t('"%webarchive" is not a valid url', array('%webarchive' => $form_state['values']['webarchive'])));
  }
}


/**
 * Save new mailing list.
 */
function mailman_manager_admin_form_submit($form, &$form_state) {
  if (!user_access('administer mailman_manager')) {
    return '';
  }

  $list->name = $form_state['values']['name'];
  $list->reqaddress = $form_state['values']['reqaddress'];
  $list->adminaddress = $form_state['values']['adminaddress'];
  $list->webaddress = $form_state['values']['webaddress'];
  $list->webarchive = $form_state['values']['webarchive'];
  $list->roles = serialize($form_state['values']['roles']);

  //update
  if (!empty($form_state['values']['lid'])) {
    $list->lid = $form_state['values']['lid'];
    drupal_write_record('mailman_lists', $list, array('lid'));
    drupal_set_message(t('Mailing List %name updated', array('%name' => $list->name)));
  }
  else {
    $lid = drupal_write_record('mailman_lists', $list);
    drupal_set_messaeg(t('New Mailman list %name successfully created', array('%name' => $list->name)));
  }
  drupal_goto('admin/settings/mailman_manager');
}